use argh::FromArgs;

static WORDS: &str = include_str!("../words.txt");

#[derive(FromArgs, PartialEq, Debug)]
/// LetterQuest solver
struct App {
    #[argh(positional)]
    query: String,
}

impl App {
    /// check if word consists only of letters in query
    fn check_word<'a>(&self, word: &'a str) -> Option<&'a str> {
        let mut query = self.query.clone();
        word.chars()
            .try_for_each(|c| {
                let position = query.chars().position(|i| i == c)?;
                query.remove(position);
                Some(())
            })
            .and(Some(word))
    }
}

fn main() {
    let app: App = argh::from_env();

    let start = std::time::Instant::now();
    let mut results = WORDS
        .lines()
        .filter_map(|line| app.check_word(line))
        .collect::<Vec<&str>>();
    eprintln!("search took: {:?}", start.elapsed());

    results.sort_by(|a, b| {
        a.len()
            .partial_cmp(&b.len())
            .expect("string length must be stortable")
    });

    results
        .iter()
        .for_each(|r| println!("matches: {} {}", r, r.len()));
}
